<p>Someone requested to reset password for {{email}}</p>
<p>Please use this link <a href="{{url}}">{{url}}</a> to reset the password.</p>
<p>This token expires after {{expires}} hours.</p>
<p>Regards,</p>
<p>The <em>{{app_name}}</em> Team</p>
